//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 整数矩形ヘッダ
 * @author Junpee
 */

#ifndef RECTANGLE_I_H_
#define RECTANGLE_I_H_

namespace Lamp{

class RectangleF;

//------------------------------------------------------------------------------
/**
 * 整数矩形
 *
 * このクラスは継承しないで下さい。
 */
class RectangleI{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// X値
			int x;
			/// Y値
			int y;
			/// 幅
			int width;
			/// 高さ
			int height;
		};

		/// 配列
		int array[4];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ矩形
	static const RectangleI zero;

	/// 単位矩形
	static const RectangleI unit;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	RectangleI(){}

	/**
	 * コンストラクタ
	 * @param sourceX X値の初期値
	 * @param sourceY Y値の初期値
	 * @param sourceWidth 幅の初期値
	 * @param sourceHeight 高さの初期値
	 */
	RectangleI(int sourceX, int sourceY, int sourceWidth, int sourceHeight) :
		x(sourceX), y(sourceY), width(sourceWidth), height(sourceHeight){}

	/**
	 * コンストラクタ
	 * @param sourceArray 初期値配列
	 */
	explicit RectangleI(int sourceArray[4]) :
		x(sourceArray[0]), y(sourceArray[1]),
		width(sourceArray[2]), height(sourceArray[3]){}

	/**
	 * コンストラクタ
	 * @param source 設定する矩形
	 */
	explicit RectangleI(const RectangleF& source);

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceX X値の設定値
	 * @param sourceY Y値の設定値
	 * @param sourceWidth 幅の設定値
	 * @param sourceHeight 高さの設定値
	 */
	inline void set(int sourceX, int sourceY, int sourceWidth, int sourceHeight){
		x = sourceX;
		y = sourceY;
		width = sourceWidth;
		height = sourceHeight;
	}

	/**
	 * 値の設定
	 * @param sourceArray 設定値配列
	 */
	inline void set(int sourceArray[4]){
		x = sourceArray[0];
		y = sourceArray[1];
		width = sourceArray[2];
		height = sourceArray[3];
	}

	/**
	 * 値の設定
	 * @param source 設定する矩形
	 */
	void set(const RectangleF& source);

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 同じ値かどうか
	 * @param target 比較する矩形
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const RectangleI& target) const{
		return ((x == target.x) && (y == target.y) &&
			(width == target.width) && (height == target.height));
	}

	/**
	 * 同じ値でないかどうか
	 * @param target 比較する矩形
	 * @return 同じ値でなければtrueを返す
	 */
	inline bool operator !=(const RectangleI& target) const{
		return ((x != target.x) || (y != target.y) ||
			(width != target.width) || (height != target.height));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 矩形の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %d, %d, %d, %d )", x, y, width, height);
		return returnString;
	}

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of RECTANGLE_I_H_
//------------------------------------------------------------------------------
